package com.axiel7.anihyou.ui.screens.mediadetails

import androidx.lifecycle.SavedStateHandle
import androidx.lifecycle.viewModelScope
import androidx.navigation.toRoute
import com.axiel7.anihyou.MediaDetailsQuery
import com.axiel7.anihyou.data.model.DataResult
import com.axiel7.anihyou.data.model.PagedResult
import com.axiel7.anihyou.data.model.stats.overview.ScoreDistribution.Companion.asStat
import com.axiel7.anihyou.data.model.stats.overview.StatusDistribution.Companion.asStat
import com.axiel7.anihyou.data.repository.FavoriteRepository
import com.axiel7.anihyou.data.repository.MediaRepository
import com.axiel7.anihyou.fragment.BasicMediaListEntry
import com.axiel7.anihyou.fragment.MediaCharacter
import com.axiel7.anihyou.type.MediaType
import com.axiel7.anihyou.ui.common.viewmodel.UiStateViewModel
import dagger.hilt.android.lifecycle.HiltViewModel
import kotlinx.coroutines.flow.launchIn
import kotlinx.coroutines.flow.onEach
import kotlinx.coroutines.flow.update
import javax.inject.Inject

@HiltViewModel
class MediaDetailsViewModel @Inject constructor(
    savedStateHandle: SavedStateHandle,
    private val mediaRepository: MediaRepository,
    private val favoriteRepository: FavoriteRepository,
) : UiStateViewModel<MediaDetailsUiState>(), MediaDetailsEvent {

    private val arguments = savedStateHandle.toRoute<MediaDetails>()

    override val initialState = MediaDetailsUiState()

    fun setIsLoggedIn(value: Boolean) {
        mutableUiState.update { it.copy(isLoggedIn = value) }
    }

    override fun onUpdateListEntry(newListEntry: BasicMediaListEntry?) {
        if (mutableUiState.value.details?.mediaListEntry?.basicMediaListEntry != newListEntry) {
            mutableUiState.update { uiState ->
                uiState.copy(
                    details = uiState.details?.copy(
                        mediaListEntry = if (newListEntry != null) {
                            uiState.details.mediaListEntry?.copy(basicMediaListEntry = newListEntry)
                                ?: MediaDetailsQuery.MediaListEntry(
                                    __typename = "MediaDetailsQuery.MediaListEntry",
                                    startedAt = null,
                                    completedAt = null,
                                    id = newListEntry.id,
                                    mediaId = uiState.details.id,
                                    basicMediaListEntry = newListEntry,
                                )
                        }
                        else null
                    )
                )
            }
        }
    }

    override fun toggleFavorite() {
        mutableUiState.value.details?.let { details ->
            favoriteRepository.toggleFavorite(
                animeId = if (details.basicMediaDetails.type == MediaType.ANIME)
                    details.id else null,
                mangaId = if (details.basicMediaDetails.type == MediaType.MANGA)
                    details.id else null,
            ).onEach { result ->
                mutableUiState.update { state ->
                    if (result is DataResult.Success && result.data != null) {
                        val newDetails = state.details
                            ?.copy(isFavourite = !state.details.isFavourite)
                            ?.also {
                                mediaRepository.updateMediaDetailsCache(it)
                            }
                        state.copy(
                            details = newDetails
                        )
                    } else {
                        state.copy(
                            error = (result as? DataResult.Error)?.message
                        )
                    }
                }
            }.launchIn(viewModelScope)
        }
    }

    override fun fetchCharactersAndStaff() {
        mediaRepository.getMediaCharactersAndStaff(mediaId = arguments.id)
            .onEach { result ->
                if (result is DataResult.Success) {
                    mutableUiState.update { uiState ->
                        uiState.copy(
                            staff = result.data.staff.map { it.mediaStaff },
                            characters = result.data.characters.map { it.mediaCharacter }
                        )
                    }
                }
            }
            .launchIn(viewModelScope)
    }

    override fun fetchRelationsAndRecommendations() {
        mediaRepository.getMediaRelationsAndRecommendations(mediaId = arguments.id)
            .onEach { result ->
                if (result is DataResult.Success) {
                    mutableUiState.update {
                        it.copy(
                            relationsAndRecommendations = result.data
                        )
                    }
                }
            }
            .launchIn(viewModelScope)
    }

    override fun fetchStats() {
        mediaRepository.getMediaStats(mediaId = arguments.id)
            .onEach { result ->
                if (result is DataResult.Success) {
                    mutableUiState.update { uiState ->
                        uiState.copy(
                            isSuccessStats = true,
                            mediaStatusDistribution = result.data?.stats?.statusDistribution
                                ?.mapNotNull { it?.asStat() }.orEmpty(),
                            mediaScoreDistribution = result.data?.stats?.scoreDistribution
                                ?.mapNotNull { it?.asStat() }.orEmpty(),
                            mediaRankings = result.data?.rankings?.filterNotNull().orEmpty()
                        )
                    }
                }
            }
            .launchIn(viewModelScope)
    }

    override fun fetchThreads() {
        mediaRepository.getMediaThreadsPage(mediaId = arguments.id, page = 1)
            .onEach { result ->
                mutableUiState.update { uiState ->
                    if (result is PagedResult.Success) {
                        uiState.copy(
                            isLoadingThreads = false,
                            threads = result.list
                        )
                    } else {
                        uiState.copy(
                            isLoadingThreads = result is PagedResult.Loading,
                        )
                    }
                }
            }
            .launchIn(viewModelScope)
    }

    override fun fetchReviews() {
        mediaRepository.getMediaReviewsPage(mediaId = arguments.id, page = 1)
            .onEach { result ->
                mutableUiState.update { uiState ->
                    if (result is PagedResult.Success) {
                        uiState.copy(
                            isLoadingReviews = false,
                            reviews = result.list
                        )
                    } else {
                        uiState.copy(
                            isLoadingThreads = result is PagedResult.Loading,
                        )
                    }
                }
            }
            .launchIn(viewModelScope)
    }

    override fun showVoiceActorsSheet(character: MediaCharacter) {
        mutableUiState.update { uiState ->
            uiState.copy(
                selectedCharacterVoiceActors = character.voiceActors?.mapNotNull { it?.commonVoiceActor },
                showVoiceActorsSheet = true
            )
        }
    }

    override fun hideVoiceActorSheet() {
        mutableUiState.update { it.copy(showVoiceActorsSheet = false) }
    }

    init {
        mediaRepository.getMediaDetails(mediaId = arguments.id)
            .onEach { result ->
                mutableUiState.update {
                    if (result is DataResult.Success) {
                        it.copy(
                            isLoading = false,
                            details = result.data
                        )
                    } else {
                        result.toUiState()
                    }
                }
            }
            .launchIn(viewModelScope)
    }
}